ObjectiveTrackerFrame.BlocksFrame.ProfessionHeader = CreateFrame("Frame", "ProfessionHeader", ObjectiveTrackerBlocksFrame, "ObjectiveTrackerHeaderTemplate");

PROFESSION_TRACKER_MODULE = ObjectiveTracker_GetModuleInfoTable();
PROFESSION_TRACKER_MODULE.updateReasonModule = OBJECTIVE_TRACKER_UPDATE_MODULE_PROFRESSION;
PROFESSION_TRACKER_MODULE.updateReasonEvents = OBJECTIVE_TRACKER_UPDATE_PROFESSION + OBJECTIVE_TRACKER_UPDATE_PROFESSION_ADDED;
PROFESSION_TRACKER_MODULE.usedBlocks = { };
PROFESSION_TRACKER_MODULE:SetHeader(ObjectiveTrackerFrame.BlocksFrame.ProfessionHeader, TRACKER_HEADER_PROFESSIONS, OBJECTIVE_TRACKER_UPDATE_PROFESSION_ADDED);

function PROFESSION_TRACKER_MODULE:OnBlockHeaderClick(block, mouseButton)
	if ( IsModifiedClick("CHATLINK") and ChatEdit_GetActiveWindow() ) then
		local professionName = GetProfessionInfo(block.id);
		local _, professionLink = GetSpellLink(professionName);

		if ( professionLink ) then
			ChatEdit_InsertLink(professionLink);
		end
	elseif ( mouseButton ~= "RightButton" ) then
		CloseDropDownMenus();

		if ( IsModifiedClick("QUESTWATCHTOGGLE") ) then
			ProfessionObjectiveTracker_UntrackProfession(_, block.id);
		else
			ProfessionObjectiveTracker_OpenProfession(_, block.id);
		end
	else
		ObjectiveTracker_ToggleDropDown(block, ProfessionObjectiveTracker_OnOpenDropDown);
	end
end

-- *****************************************************************************************************
-- ***** BLOCK DROPDOWN FUNCTIONS
-- *****************************************************************************************************

function ProfessionObjectiveTracker_OnOpenDropDown(self)
	local block = self.activeFrame;
	local professionName = GetProfessionInfo(block.id);
	local _, professionLink = GetSpellLink(professionName);

	local info = UIDropDownMenu_CreateInfo();
	info.text = professionName;
	info.isTitle = 1;
	info.notCheckable = true;
	UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL);

	if ( professionLink ) then
		local info = UIDropDownMenu_CreateInfo();
		info.text = OBJECTIVES_VIEW_PROFESSION;
		info.func = ProfessionObjectiveTracker_OpenProfession;
		info.arg1 = block.id;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL);

		local info = UIDropDownMenu_CreateInfo();
		info.text = OBJECTIVES_TRACK_COOLDOWNS;
		info.func = ProfessionObjectiveTracker_ToggleCooldowns;
		info.arg1 = block.id;
		info.checked = ( trackedCooldown[block.id] );
		UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL);

		local info = UIDropDownMenu_CreateInfo();
		info.text = OBJECTIVES_TRACK_REAGENTS;
		info.func = ProfessionObjectiveTracker_ToggleReagents;
		info.arg1 = block.id;
		info.checked = ( trackedCooldown[block.id] == 1 );
		info.disabled = ( not trackedCooldown[block.id] );
		UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL);
	end

	local info = UIDropDownMenu_CreateInfo();
	info.text = OBJECTIVES_STOP_TRACKING;
	info.func = ProfessionObjectiveTracker_UntrackProfession;
	info.arg1 = block.id;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL);
end

function ProfessionObjectiveTracker_OpenProfession(dropDownButton, professionID)
	local professionName = GetProfessionInfo(professionID);
	local _, professionLink = GetSpellLink(professionName);

	if ( professionLink ) then
		CastSpellByName(professionName, "player");
	end
end

function ProfessionObjectiveTracker_ToggleCooldowns(dropDownButton, professionID)
	if ( not trackedCooldown[professionID] ) then
		trackedCooldown[professionID] = 1;
	else
		trackedCooldown[professionID] = nil;
	end

	ObjectiveTracker_Update(OBJECTIVE_TRACKER_UPDATE_PROFESSION);
end

function ProfessionObjectiveTracker_ToggleReagents(dropDownButton, professionID)
	if ( trackedCooldown[professionID] == 0 ) then
		trackedCooldown[professionID] = 1;
	else
		trackedCooldown[professionID] = 0;
	end

	ObjectiveTracker_Update(OBJECTIVE_TRACKER_UPDATE_PROFESSION);
end

function ProfessionObjectiveTracker_UntrackProfession(dropDownButton, professionID)
	RemoveTrackedProfession(professionID);

	if ( SpellBookProfessionFrame ) then
		for index = 1, 2 do
			local frame = _G["PrimaryProfession".. index];

			if ( IsTrackedProfession(frame.professionID) ) then
				frame.professionName:SetTextColor(0, 0.82, 1);
			else
				frame.professionName:SetTextColor(1, 0.82, 0);
			end
		end

		for index = 1, 4 do
			local frame = _G["SecondaryProfession".. index];

			if ( IsTrackedProfession(frame.professionID) ) then
				frame.professionName:SetTextColor(0, 0.82, 1);
			else
				frame.professionName:SetTextColor(1, 0.82, 0);
			end
		end
	end

	ObjectiveTracker_Update(OBJECTIVE_TRACKER_UPDATE_PROFESSION);
end

-- *****************************************************************************************************
-- ***** UPDATE FUNCTIONS
-- *****************************************************************************************************

function PROFESSION_TRACKER_MODULE:Update()
	PROFESSION_TRACKER_MODULE:BeginLayout();

	for key, professionID in pairs(trackedProfession) do
		if ( professionID ) then
			local professionName, _, skillLevel, maxSkillLevel = GetProfessionInfo(professionID);
			local block = PROFESSION_TRACKER_MODULE:GetBlock(professionID);

			PROFESSION_TRACKER_MODULE:SetBlockHeader(block, professionName .." (".. skillLevel .."/".. maxSkillLevel ..")");

			if ( trackedCooldown[professionID] and trackedTradeSkill[professionName] ) then
				for skillID, skill in pairs(trackedTradeSkill[professionName]) do
					if ( not skill.cooldown or (skill.datetime + skill.cooldown) <= time() ) then
						local hasReagent = true;

						for reagentID, reagent in pairs(skill.reagents) do
							if ( not trackedReagent[reagentID] or reagent.count > trackedReagent[reagentID] ) then
								hasReagent = false;
							end
						end

						if ( hasReagent ) then
							PROFESSION_TRACKER_MODULE:AddObjective(block, skillID, skill.name, nil, nil, nil, OBJECTIVE_TRACKER_COLOR["Normal"]);
						elseif ( trackedCooldown[professionID] == 0 ) then
							PROFESSION_TRACKER_MODULE:AddObjective(block, skillID, skill.name, nil, nil, nil, OBJECTIVE_TRACKER_COLOR["Failed"]);
						end
					end
				end
			end

			block:SetHeight(block.height);

			if ( ObjectiveTracker_AddBlock(block) ) then
				block:Show();
				PROFESSION_TRACKER_MODULE:FreeUnusedLines(block);
			else
				block.used = false;
				break;
			end
		end
	end

	PROFESSION_TRACKER_MODULE:EndLayout();
end
